package MT::Plugin::SKR::CategoryMaker;

use strict;
use MT::Category;
use YAML::Tiny;
use Text::CSV_PP;

use vars qw( $NAME $VERSION );
($NAME) = __PACKAGE__ =~ /.+::(\w+)/;
$VERSION = '0.10';

use base qw( MT::Plugin );
my $plugin = __PACKAGE__->new ({
    id => $NAME,
    key => $NAME,
    name => $NAME,
    version => $VERSION,
    author_name => 'SKYARC System Co.,Ltd.',
    author_link => 'http://www.skyarc.co.jp/',
    doc_link => 'http://www.skyarc.co.jp/engineerblog/entry/categorymaker.html',
    description => <<HTMLHEREDOC,
<__trans phrase="Create categories and folders at once with YAML directives.">
HTMLHEREDOC
    # for Configuration
    blog_config_template => \&_config_tmpl,
    settings => new MT::PluginSettings([
        ['yaml', { Default => undef }],
    ]),
    # Localization
    l10n_class => $NAME. '::L10N',
});
MT->add_plugin ($plugin);

###
sub _config_tmpl {
    <<'HTMLHEREDOC';
<mtapp:setting
    id="yaml"
    label="<__trans phrase="Categories/Folders">"
    hint="<__trans phrase="Set the directives with YAML">"
    show_hint="1">
<textarea name="yaml" rows="9"><$mt:var name="yaml" escape="html"$></textarea>
</mtapp:setting>
HTMLHEREDOC
}

### save_config
sub save_config {
    my ($plugin, $param, $scope) = @_;

    if (my $yaml = $param->{yaml}) {
        my ($blog_id) = $scope =~ /blog:(\d+)/;
        eval {
            $yaml = YAML::Tiny::Load ("$yaml\n"); # for safe
            # Categories
            _create_category ($blog_id, 'category', undef,
                    $yaml->{categories} || $yaml->{category} || []);
            # Folders
            _create_category ($blog_id, 'folder', undef,
                    $yaml->{folders} || $yaml->{folder} || []);
            $param->{yaml} = undef; # success
        };
        MT->log ("$NAME: $@") if $@;
    }

    $plugin->SUPER::save_config ($param, $scope);
}

###
sub _create_category {
    my ($blog_id, $class, $parent_cat, $yaml) = @_;

    my $csv = Text::CSV_PP->new ({binary => 1});

    foreach (@$yaml) {
        my ($label, $child) = ($_, undef);
        if (ref $_ eq 'HASH') {
            my @keys = keys %$_;
            $label = $keys[0];
            $child = $_->{$label};
        }

        $csv->parse ($label);
        my @columns = $csv->fields;

        my $cat = MT::Category->new;
        $cat->blog_id ($blog_id);
        $cat->class ($class);
        $cat->label ($columns[0]);
        $cat->basename ($columns[1] || $columns[0]);
        $cat->description ($columns[2] || $columns[0]);
        $cat->parent ($parent_cat ? $parent_cat->id : 0);
        $cat->save;

        _create_category ($blog_id, $class, $cat, $child) if $child; # recursive
    }
}

1;